require 'src/utils'
require 'src/assets'
require 'src/draw'
require 'src/collisions'
require 'src/physics'
require 'src/controls'
require 'src/socktest'
require 'src/rigs'

imageDefs = assets.imageDefs
sounds = assets.sounds
--playsound = TEsound.play
playsound = function() end

local drawMethods = draw.drawMethods
local ctrlState = controls.ctrlState 
local beginContact = physics.beginContact
local endContact = physics.endContact
local preSolve = physics.preSolve
local postSolve = physics.postSolve
local updateRigs = rigeng.updateRigs

--socktest.nodeTcpTest()

----------------------------------------------

local level = {  
	drawables = {},
	activeRigs = {}
}
cLevel = level

----------------------------------------------

function love.load()
	assets.loadAndPrepImages()
	cLevel:init()
end

local skipfirstfew = 0
local numUpdates = 0
local mem = 0
local lastMem = 0
function love.update(dt)
	if skipfirstfew > 10 then  	-- to skip long dt's at startup
		cLevel:update(dt)
		numUpdates = numUpdates + 1
	else 
		skipfirstfew = skipfirstfew + 1
	end

	TEsound.cleanup()

	collectgarbage()
	--[[
	mem = collectgarbage('count') -- "collect" "step" "setpause" "setstepmul
	if (mem > lastMem) then
		print('mem '..mem..'   numUpdates '..numUpdates)
	end
	--]]
end

function love.draw()
	cLevel:draw()
end

----------------------------------------------

function level:init() 
	self.showPhyShapes = true

	self.world = love.physics.newWorld(0, 50)
	self.world:setCallbacks(beginContact, endContact, preSolve, postSolve)

	self.ground = rigs.ground(self)
	self.clouds = rigs.clouds(nil, 5, self)

	local saucy = rigs.saucer(nil, 200, 150, self)
	local gun = rigs.gun(nil, 200, 300, self)
	ctrlState.ctrlRigs[#ctrlState.ctrlRigs + 1] = saucy
	ctrlState.ctrlRigs[#ctrlState.ctrlRigs + 1] = gun

	self.explosions = rigs.explosions(self)
	print 'init done'
end

function level:update(dt)
	if ctrlState.mode ~= 'paused' then
		controls.checkHeldKeys(dt)
		controls.update(dt)
		updateRigs(self.activeRigs, dt, self)
		self.world:update(dt)
	end
end

function level:draw()
	for i, v in ipairs(self.drawables) do
		v.draw(v, level)
	end
end

--------------------------------------

-- a volcano erupts every 2 minutes
function level_with_volcano( interface )
   while true do
      wait(seconds(5))
      start_eruption_volcano()
      wait(frames(10))
      s = play("rumble_sound")
      wait( end_of(s) )
      start_camera_shake()
      -- more stuff
      wait(minutes(2))
    end
end

-- The main program (in C or Lua) creates a coroutine to run level_with_volcano.
--  The end_of, frames, seconds, and minutes functions return a special type 
--  that wait understands. The wait function uses that special type to decide 
--  how long the coroutine needs to sleep and yields that value to the main 
--  program. The main program goes on about its 
--  business (drawing, updating, etc) and after enough time has passed
--   it resumes the level_with_volcano coroutine

