local callCoros = utils.callCoros

module('controls', package.seeall)

ctrlState = {
	global = {},
	ctrlRigs = {},
	coroEvntHndlrsNeedingUpdates = {},
	mode = 'normal'
}
local coroFuns = ctrlState.coroEvntHndlrsNeedingUpdates

function make(info, inpMethod, val, fun)
	return {info = info, inpMethod = inpMethod, val = val, fun = fun}
end

function makeco(info, inpMethod, val, fun)
	return {info = info, inpMethod = inpMethod, val = val, fun = fun, isCoro=true}
end

--------------------------------
-- World:rayCast( x1, y1, x2, y2, callback )
-- callback(fixture, x, y, xn, yn, fraction) -- return continuing ray length, 0 terminates

-- World:queryBoundingBox( topLeftX, topLeftY, bottomRightX, bottomRightY, callback )
-- callback(fixture) -- returns true to continue, false not
function findClickedRig(mx, my)
	local rig, blk

	function bbCallback(fixt)
		print('type fixt '..type(fixt))
		local t = fixt:getUserData()
		if type(t) == 'table' then
    		print('under pick - rig: '..t.rig.name..' blk: '..t.blk.name)
    		rig = t.rig
    		blk = t.blk
    	else 
    		print('userdata was '..type(t)..'   huh?')
    	end
		return true
	end

	function rayCallback(fixt)
		print('type fixt '..type(fixt))
		local t = fixt:getUserData()
		if type(t) == 'table' then
    		print('under pick - rig: '..t.rig.name..' blk: '..t.blk.name)
    		rig = t.rig
    		blk = t.blk
    	else 
    		print('userdata was '..type(t)..'   huh?')
    	end
		return 0
	end

	cLevel.world:queryBoundingBox(mx, my, mx, my, bbCallback)
	--cLevel.world:rayCast( mx, my, mx+20, my, rayCallback )
	return rig, blk
end

function rigClicked(r,b,mx,my)
	print('clicked rig '..r.name..'   blk '..b.name)
end

---------------------------------

do
	local fx = 100
	local fy = 100
	local torque = 3000

	basicControls = {
		make('force left', 'keyheld', 'a', function(r, dt)
			r.base.b:applyForce(-fx*r.base.mass, 0)
		end),
		make('force right', 'keyheld', 'd', function(r, dt)
			r.base.b:applyForce(fx*r.base.mass, 0)
		end),
		make('force up', 'keyheld', 'w', function(r, dt)
			r.base.b:applyForce(0, -fy*r.base.mass)
		end),
		make('force down', 'keyheld', 's', function(r, dt)
			r.base.b:applyForce(0, fy*r.base.mass)
		end),	
		make('rotate left', 'keyheld', 'q', function(r, dt)
			r.base.b:applyTorque(-torque*r.base.mass)
		end),
		make('rotate right', 'keyheld', 'e', function(r, dt)
			r.base.b:applyTorque(torque*r.base.mass)
		end)
	}
end

ctrlState.global.normal = {
	make('test fountain', 'key', '1', function()
		rigs.block_fountain(nil, love.mouse.getX(), love.mouse.getY(), cLevel)
	end),
	makeco('start mouse', 'mouse', 'any', function(dt, ctrltbl, mx, my, button)
		if (button=='ld') then
			local r,b = findClickedRig(mx, my)
			if r then
				rigClicked(r,b,mx,my)
				return
			end
			-- starting camera pan
			coroFuns[ctrltbl] = ctrltbl.coro
			while true do
				-- note: if dt then ctrltbl and button will be nil 
				-- i.e. call came from update() not input event
				dt,ctrltbl,mx,my,button = coroutine.yield()
				if (dt) then
					print('in drag loop   dt '..dt..'   mx '..mx..'   my '..my)
				elseif button == 'lu' then
					print('ending mouse drag')
					coroFuns[ctrltbl] = nil
					return
				end			
			end
		end
	end),	
	make('test shower', 'mouse', 'rd', function(mx, my)
		rigs.block_shower(nil, mx, my, 8, cLevel)
	end),	
	make('pause', 'key', 'p', function()
		ctrlState.mode = 'paused'
	end)
}

ctrlState.global.paused = {
	make('unpause', 'key', 'p', function()
		print'unpause'
		ctrlState.mode = 'normal' 
	end)
}

-------------------------------------------------------------------

function callCtrlFun(c, ...)
	if c.isCoro then
		if not c.coro or coroutine.status(c.coro) == 'dead' then
			c.coro = coroutine.create(c.fun)
		end
		-- note: nil param indicates from event (no dt from update)
		local status,err = coroutine.resume(c.coro, nil, c, ...) 
		if status == false then
			local stacktrace = debug.traceback(c.coro)
			error('coroutine error.\n ctrl: '..c.info..' inpMethod: '..c.inpMethod
				..'\n err: '..err..'\n '..stacktrace)
		end
	else
		c.fun(...)
	end
end

function routeCtrlGroup(grp, inpMethod, val, ...)
	for i,c in ipairs(grp) do
		if c.inpMethod == inpMethod then
			if inpMethod == 'keyheld' then
				if love.keyboard.isDown(c.val) then
					callCtrlFun(c, ...) 
				end
			else 
				if c.val=='any' or c.val == val then
					callCtrlFun(c, ...)
				end
			end
		end
	end
end

function routeControls(inpMethod, val, ...)
	routeCtrlGroup(ctrlState.global[ctrlState.mode], inpMethod, val, ...)
	for i,r in ipairs(ctrlState.ctrlRigs) do
		if r.ctrls and r.ctrls[ctrlState.mode] then
			routeCtrlGroup(r.ctrls[ctrlState.mode], inpMethod, val, r, ...)
		end
	end
end

function love.keypressed( key, unicode )
	routeControls('key', key)
end

function love.mousepressed(mx, my, button)
	local b = button..'d'
	routeControls('mouse', b, mx, my, b)
end

function love.mousereleased(mx, my, button)
	local b = button..'u'
	routeControls('mouse', b, mx, my, b)
end

function checkHeldKeys(dt)
	routeControls('keyheld', nil, dt)
end

function update(dt)
	function onCoroError(err, stacktrace, k, dt)
		error('coroutine error.\n key: '..k
				..'\n err: '..err..'\n '..stacktrace)
	end

	callCoros(ctrlState.coroEvntHndlrsNeedingUpdates, onCoroError, dt, nil, love.mouse.getPosition())
end

