local drawMethods = draw.drawMethods
local physet = physics.physet
local callCoros = utils.callCoros

module('rigeng', package.seeall)


function updateRigs(rigs, dt, level)
	local status,err
	for r in pairs(rigs) do
		for k, u in pairs(r.updateFuns) do
			u(r, dt, level)
		end

		function onCoroError(err, stacktrace, k, r, dt)
			error('coroutine error.\n rig: '..r.name..' key: '..k
					..'\n err: '..err..'\n '..stacktrace)
		end

		callCoros(r.updateCoFuns, onCoroError, r, dt)
	end
end

-------------------------------------

function addBlk(rig, def)

	local blk = {}
	rig.blks[blk] = 1
	rig.num = rig.num + 1

	-- shallow copy, replace functions with retval
	for k,v in pairs(def) do
		if (type(v)=='function') then
			blk[k] = v(rig, blk, k)
		else 
			blk[k] = v
		end
	end

	table.insert(rig.lvl.drawables, blk)

	if blk.phyty then
		physet[blk.phyty](rig, blk)
		blk.mass = blk.b:getMass()
		blk.b:setLinearVelocity(blk.vx or 0, blk.vy or 0) 
	else
		blk.draw = drawMethods.plainImage
	end

	return blk	
end

function removeBlk(rig, v) 
	rig.blks[v] = nil
	rig.num = rig.num - 1
	if v.b then 
		v.b:destroy() 
		v.b = nil
	end
	local vi
	for i,d in ipairs(rig.lvl.drawables) do
		if d==v then vi=i; break end
	end
	table.remove(rig.lvl.drawables, vi)
	if rig.num < 0 then cause_error() end
	if rig.num == 0 and rig.removeOnEmpty then
		rig.lvl.activeRigs[rig] = nil
	end
end

-------------------------------------

function timedLife(rig, dt)
	for v in pairs(rig.blks) do
		v.duration = v.duration - dt
		if v.duration < 0 then
			removeBlk(rig, v)
		end
	end
end

function spawner(rig, dt)
	rig.tNextSpawn = rig.tNextSpawn - dt
	if (rig.tNextSpawn <= 0 and rig.num < rig.maxNum) then
		rig.tNextSpawn = rig.spawnTime
		addBlk(rig, rig.blkDef)		
	end
end

-------------------------------------------

function basicRig(level, name)
	local r = { 
		lvl = level,
		blks = {}, 
		num = 0,
		updateFuns = {},
		updateCoFuns = {},
		removeOnEmpty = false,
		ctrls = {},
		name = name or 'unnamed',
	}
	level.activeRigs[r] = 1
	return r
end