require 'src/rigeng'

local addBlk = rigeng.addBlk
local removeBlk = rigeng.removeBlk
local timedLife = rigeng.timedLife
local spawner = rigeng.spawner
local basicRig = rigeng.basicRig
local collisionmap = collisions.collisionmap
local collisionfns = collisions.collisionfns
local basicControls = controls.basicControls

module('rigs', package.seeall)

rigs.block_shower = function (r, mx, my, num, level)
	r = r or basicRig(level, 'block shower')
	r.removeOnEmpty = true

	r.blkDef = {
		imgDef = imageDefs.green,
		phyty = 'dynamicCircle',
		name = 'ball'
	}

	for i = 0, num do
		r.blkDef.x = mx + math.random(-200, 200)
		r.blkDef.y = my + math.random(-40, 40)
		r.blkDef.vx = math.random(-5,5)
		r.blkDef.vy = math.random(-5,5)
		r.blkDef.duration = i * 0.3
		addBlk(r, r.blkDef)
	end

	r.updateFuns['time_em'] = timedLife
	return r	
end

rigs.block_fountain = function (r, mx, my, level)
	r = r or basicRig(level, 'block fountain')
	r.spawnTime = 0.3
	r.tNextSpawn = r.spawnTime
	r.maxNum = 20

	r.blkDef = {
		imgDef = imageDefs.green,
		phyty = 'dynamicCircleNoSelfCollide',
		name = 'ball',
		duration = 2,
		x = mx,
		y = my,
		vx = function() return math.random(-200,200) end,
		vy = function() return math.random(200,800) end,
	}

	r.updateFuns['spawn_em'] = spawner
	r.updateFuns['time_em'] = timedLife	
	return r	
end

rigs.clouds = function (r, num, level)
	r = r or basicRig(level, 'clouds')

	cloudImgs = {
		imageDefs.cloud1,
		imageDefs.cloud2
	}

	r.blkDef = {
		name = cloud,
		imgDef = function()
			return cloudImgs[math.random(1, #cloudImgs)]
		end,
		x = function() return math.random(-50, 200) end,
		y = function() return math.random(-100, 70) end,
		dx = function() return math.random(5, 200) end,
	}

	for i = 0, num do
		addBlk(r, r.blkDef)
	end

	r.updateFuns['move_clouds'] = function(rig, dt, level)
		for b in pairs(rig.blks) do
			b.x = b.x + b.dx*dt
			if (b.x > 1000) then b.x = -100 end
		end
	end

	return r	
end

rigs.saucer = function (r, x, y, level)
	r = r or basicRig(level, 'saucer')
	
	r.blkDef = {
		name = 'saucer',
		imgDef = imageDefs.saucer,
		phyty = 'dynamicRect',
		x = x,
		y = y,
	}

	r.base = addBlk(r, r.blkDef)

	r.ctrls.normal = basicControls

	local gx, gy = level.world:getGravity()
	r.updateFuns['defy gravity'] = function(rig, dt)
		for b in pairs(rig.blks) do
			b.b:applyForce(b.b:getMass() * -gx, b.b:getMass() * -gy)
		end
	end

	r.updateCoFuns['fly around'] = coroutine.create(function(rig, dt)
		
		local period = 3  			-- time for one cycle (2 pi)
		local t = period / 4		-- start time
		local mf = 50				-- max force to apply
		local a, f					-- sin(w) sould range 1,-1 over period
		while true do
			t = t + dt			-- current time in seconds?
			a = (t/period) * 2 * math.pi
			f = math.sin(a) * mf
			--print('dt '..dt..'     t '..t..'     a '..a..'    f '..f)
			for b in pairs(rig.blks) do
				b.b:applyLinearImpulse(f, 0)
				--b.b:applyForce(f, 0)
			end

			if (dt > 0.1) then
				print('whoa, big dt '..dt)
			end

			_, dt = coroutine.yield() -- note: _ is rig
		end
	end)

	return r	
end

rigs.gunshots = function (r, gunrig, level)
	r = r or basicRig(level, 'gunshots')
	r.gunrig = gunrig
	r.shotDef = { 
		name = 'gunshot',	
		duration = 3,
		imgDef = imageDefs.eshot,
		phyty = 'dynamicCircleNoSelfCollide',
		x = function(rig) return rig.gunrig.gunbase.b:getX() end,
		y = function(rig) return rig.gunrig.gunbase.b:getY() end,
		vx = function(rig) return 200 end,
		vy = function(rig) return -300 end,
	}
	function r:shoot()
		addBlk(self, self.shotDef);
	end
	r.updateFuns['time_em'] = timedLife
	return r
end

rigs.gun = function (r, x, y, level)
	r = r or basicRig(level, 'gun')
	
	r.gunshots = rigs.gunshots(nil, r, level)

	r.blkDef = {
		name = 'gunbase',
		imgDef = imageDefs.turretbase,
		--phyty = 'kinematicRect',
		phyty = 'dynamicRect',
		x = x,
		y = y,
	}

	--r.base = addBlk(r, r.blkDef)
	r.gunbase = addBlk(r, r.blkDef);

	r.ctrls.normal = {
		controls.make('shoot', 'key', ' ', function()
			r.gunshots:shoot()
			playsound(sounds.shoot1)
		end)
	}

	--body->ApplyForce( body->GetMass() * -m_world->GetGravity(), body->GetWorldCenter() );
	--Body:applyForce( fx, fy, x, y )   -- x,y default to center of mass
	--Body:applyLinearImpulse( ix, iy, x, y )

	-- local gx, gy = level.world:getGravity()
	-- r.updaters['defy gravity'] = function(rig, dt)
	-- 	for b in pairs(rig.blks) do
	-- 		b.b:applyForce(b.b:getMass() * -gx, b.b:getMass() * -gy)
	-- 	end
	-- end

	return r	
end

rigs.explosions = function (level)
	r = basicRig(level, 'explosions')

	r.blkDef = {
		name = 'firepuff',
		imgDef = imageDefs.explode1,
		duration = 1.5,
	}	

	function r:addExplosion(x,y)
		self.blkDef.x = x
		self.blkDef.y = y
		addBlk(self, self.blkDef);
	end

	r.updateFuns['time_em'] = timedLife

	return r
end

rigs.ground = function (level)
	r = basicRig(level, 'ground')

	groundImgs = {
		imageDefs.greybox,
		imageDefs.brownbox
	}
	r.blkDef = {
		name = 'groundblock',
		imgDef = function()
			return groundImgs[math.random(1, #groundImgs)]
		end,
		phyty = 'staticRect',
	}	

	r.mainbody = love.physics.newBody(level.world, 0, 0, "static")

	for i = 0, 20 do
		r.blkDef.x = i*32
		r.blkDef.y = math.sin(i/5)*20+550
		addBlk(r, r.blkDef)
	end		
end
